@@ -84,8 +84,8 @@ ast_enum_of_structs! {
             pub attrs: Vec<Attribute>,
             pub vis: Visibility,
             pub constness: Option<Token![const]>,
-            pub unsafety: Option<Token![unsafe]>,
             pub asyncness: Option<Token![async]>,
+            pub unsafety: Option<Token![unsafe]>,
             pub abi: Option<Abi>,
             pub ident: Ident,
             pub decl: Box<FnDecl>,
@@ -687,8 +687,8 @@ ast_struct! {
     /// *This type is available if Syn is built with the `"full"` feature.*
     pub struct MethodSig {
         pub constness: Option<Token![const]>,
-        pub unsafety: Option<Token![unsafe]>,
         pub asyncness: Option<Token![async]>,
+        pub unsafety: Option<Token![unsafe]>,
         pub abi: Option<Abi>,
         pub ident: Ident,
         pub decl: FnDecl,
@@ -767,12 +767,12 @@ pub mod parsing {
 
     impl Parse for Item {
         fn parse(input: ParseStream) -> Result<Self> {
+            let mut attrs = input.call(Attribute::parse_outer)?;
             let ahead = input.fork();
-            ahead.call(Attribute::parse_outer)?;
             let vis: Visibility = ahead.parse()?;
 
             let lookahead = ahead.lookahead1();
-            if lookahead.peek(Token![extern]) {
+            let mut item = if lookahead.peek(Token![extern]) {
                 ahead.parse::<Token![extern]>()?;
                 let lookahead = ahead.lookahead1();
                 if lookahead.peek(Token![crate]) {
@@ -819,7 +819,7 @@ pub mod parsing {
                     || lookahead.peek(Token![auto]) && ahead.peek2(Token![trait])
                 {
                     input.parse().map(Item::Trait)
-                } else if lookahead.peek(Token![impl ]) {
+                } else if lookahead.peek(Token![impl]) {
                     input.parse().map(Item::Impl)
                 } else if lookahead.peek(Token![async])
                     || lookahead.peek(Token![extern])
@@ -847,7 +847,7 @@ pub mod parsing {
                 input.call(parse_trait_or_trait_alias)
             } else if lookahead.peek(Token![auto]) && ahead.peek2(Token![trait]) {
                 input.parse().map(Item::Trait)
-            } else if lookahead.peek(Token![impl ])
+            } else if lookahead.peek(Token![impl])
                 || lookahead.peek(Token![default]) && !ahead.peek2(Token![!])
             {
                 input.parse().map(Item::Impl)
@@ -864,7 +864,34 @@ pub mod parsing {
                 input.parse().map(Item::Macro)
             } else {
                 Err(lookahead.error())
+            }?;
+
+            {
+                let item_attrs = match item {
+                    Item::ExternCrate(ref mut item) => &mut item.attrs,
+                    Item::Use(ref mut item) => &mut item.attrs,
+                    Item::Static(ref mut item) => &mut item.attrs,
+                    Item::Const(ref mut item) => &mut item.attrs,
+                    Item::Fn(ref mut item) => &mut item.attrs,
+                    Item::Mod(ref mut item) => &mut item.attrs,
+                    Item::ForeignMod(ref mut item) => &mut item.attrs,
+                    Item::Type(ref mut item) => &mut item.attrs,
+                    Item::Existential(ref mut item) => &mut item.attrs,
+                    Item::Struct(ref mut item) => &mut item.attrs,
+                    Item::Enum(ref mut item) => &mut item.attrs,
+                    Item::Union(ref mut item) => &mut item.attrs,
+                    Item::Trait(ref mut item) => &mut item.attrs,
+                    Item::TraitAlias(ref mut item) => &mut item.attrs,
+                    Item::Impl(ref mut item) => &mut item.attrs,
+                    Item::Macro(ref mut item) => &mut item.attrs,
+                    Item::Macro2(ref mut item) => &mut item.attrs,
+                    Item::Verbatim(_) => unreachable!(),
+                };
+                attrs.extend(item_attrs.drain(..));
+                *item_attrs = attrs;
             }
+
+            Ok(item)
         }
     }
 
@@ -984,56 +1011,58 @@ pub mod parsing {
                 vis: input.parse()?,
                 use_token: input.parse()?,
                 leading_colon: input.parse()?,
-                tree: input.call(use_tree)?,
+                tree: input.parse()?,
                 semi_token: input.parse()?,
             })
         }
     }
 
-    fn use_tree(input: ParseStream) -> Result<UseTree> {
-        let lookahead = input.lookahead1();
-        if lookahead.peek(Ident)
-            || lookahead.peek(Token![self])
-            || lookahead.peek(Token![super])
-            || lookahead.peek(Token![crate])
-            || lookahead.peek(Token![extern])
-        {
-            let ident = input.call(Ident::parse_any)?;
-            if input.peek(Token![::]) {
-                Ok(UseTree::Path(UsePath {
-                    ident: ident,
-                    colon2_token: input.parse()?,
-                    tree: Box::new(input.call(use_tree)?),
+    impl Parse for UseTree {
+        fn parse(input: ParseStream) -> Result<UseTree> {
+            let lookahead = input.lookahead1();
+            if lookahead.peek(Ident)
+                || lookahead.peek(Token![self])
+                || lookahead.peek(Token![super])
+                || lookahead.peek(Token![crate])
+                || lookahead.peek(Token![extern])
+            {
+                let ident = input.call(Ident::parse_any)?;
+                if input.peek(Token![::]) {
+                    Ok(UseTree::Path(UsePath {
+                        ident: ident,
+                        colon2_token: input.parse()?,
+                        tree: Box::new(input.parse()?),
+                    }))
+                } else if input.peek(Token![as]) {
+                    Ok(UseTree::Rename(UseRename {
+                        ident: ident,
+                        as_token: input.parse()?,
+                        rename: {
+                            if input.peek(Ident) {
+                                input.parse()?
+                            } else if input.peek(Token![_]) {
+                                Ident::from(input.parse::<Token![_]>()?)
+                            } else {
+                                return Err(input.error("expected identifier or underscore"));
+                            }
+                        },
+                    }))
+                } else {
+                    Ok(UseTree::Name(UseName { ident: ident }))
+                }
+            } else if lookahead.peek(Token![*]) {
+                Ok(UseTree::Glob(UseGlob {
+                    star_token: input.parse()?,
                 }))
-            } else if input.peek(Token![as]) {
-                Ok(UseTree::Rename(UseRename {
-                    ident: ident,
-                    as_token: input.parse()?,
-                    rename: {
-                        if input.peek(Ident) {
-                            input.parse()?
-                        } else if input.peek(Token![_]) {
-                            Ident::from(input.parse::<Token![_]>()?)
-                        } else {
-                            return Err(input.error("expected identifier or underscore"));
-                        }
-                    },
+            } else if lookahead.peek(token::Brace) {
+                let content;
+                Ok(UseTree::Group(UseGroup {
+                    brace_token: braced!(content in input),
+                    items: content.parse_terminated(UseTree::parse)?,
                 }))
             } else {
-                Ok(UseTree::Name(UseName { ident: ident }))
+                Err(lookahead.error())
             }
-        } else if lookahead.peek(Token![*]) {
-            Ok(UseTree::Glob(UseGlob {
-                star_token: input.parse()?,
-            }))
-        } else if lookahead.peek(token::Brace) {
-            let content;
-            Ok(UseTree::Group(UseGroup {
-                brace_token: braced!(content in input),
-                items: content.parse_terminated(use_tree)?,
-            }))
-        } else {
-            Err(lookahead.error())
         }
     }
 
@@ -1082,8 +1111,8 @@ pub mod parsing {
             let outer_attrs = input.call(Attribute::parse_outer)?;
             let vis: Visibility = input.parse()?;
             let constness: Option<Token![const]> = input.parse()?;
-            let unsafety: Option<Token![unsafe]> = input.parse()?;
             let asyncness: Option<Token![async]> = input.parse()?;
+            let unsafety: Option<Token![unsafe]> = input.parse()?;
             let abi: Option<Abi> = input.parse()?;
             let fn_token: Token![fn] = input.parse()?;
             let ident: Ident = input.parse()?;
@@ -1092,6 +1121,13 @@ pub mod parsing {
             let content;
             let paren_token = parenthesized!(content in input);
             let inputs = content.parse_terminated(FnArg::parse)?;
+            let variadic: Option<Token![...]> = match inputs.last() {
+                Some(punctuated::Pair::End(&FnArg::Captured(ArgCaptured {
+                    ty: Type::Verbatim(TypeVerbatim { ref tts }),
+                    ..
+                }))) => parse2(tts.clone()).ok(),
+                _ => None,
+            };
 
             let output: ReturnType = input.parse()?;
             let where_clause: Option<WhereClause> = input.parse()?;
@@ -1105,8 +1141,8 @@ pub mod parsing {
                 attrs: private::attrs(outer_attrs, inner_attrs),
                 vis: vis,
                 constness: constness,
-                unsafety: unsafety,
                 asyncness: asyncness,
+                unsafety: unsafety,
                 abi: abi,
                 ident: ident,
                 decl: Box::new(FnDecl {
@@ -1114,7 +1150,7 @@ pub mod parsing {
                     paren_token: paren_token,
                     inputs: inputs,
                     output: output,
-                    variadic: None,
+                    variadic: variadic,
                     generics: Generics {
                         where_clause: where_clause,
                         ..generics
@@ -1130,6 +1166,8 @@ pub mod parsing {
 
     impl Parse for FnArg {
         fn parse(input: ParseStream) -> Result<Self> {
+            // TODO: optimize using advance_to
+
             if input.peek(Token![&]) {
                 let ahead = input.fork();
                 if ahead.call(arg_self_ref).is_ok() && !ahead.peek(Token![:]) {
@@ -1179,7 +1217,23 @@ pub mod parsing {
         Ok(ArgCaptured {
             pat: input.parse()?,
             colon_token: input.parse()?,
-            ty: input.parse()?,
+            ty: match input.parse::<Token![...]>() {
+                Ok(dot3) => {
+                    let args = vec![
+                        TokenTree::Punct(Punct::new('.', Spacing::Joint)),
+                        TokenTree::Punct(Punct::new('.', Spacing::Joint)),
+                        TokenTree::Punct(Punct::new('.', Spacing::Alone)),
+                    ];
+                    let tokens = TokenStream::from_iter(args.into_iter().zip(&dot3.spans).map(
+                        |(mut arg, span)| {
+                            arg.set_span(*span);
+                            arg
+                        },
+                    ));
+                    Type::Verbatim(TypeVerbatim { tts: tokens })
+                }
+                Err(_) => input.parse()?,
+            },
         })
     }
 
@@ -1248,12 +1302,12 @@ pub mod parsing {
 
     impl Parse for ForeignItem {
         fn parse(input: ParseStream) -> Result<Self> {
+            let mut attrs = input.call(Attribute::parse_outer)?;
             let ahead = input.fork();
-            ahead.call(Attribute::parse_outer)?;
             let vis: Visibility = ahead.parse()?;
 
             let lookahead = ahead.lookahead1();
-            if lookahead.peek(Token![fn]) {
+            let mut item = if lookahead.peek(Token![fn]) {
                 input.parse().map(ForeignItem::Fn)
             } else if lookahead.peek(Token![static]) {
                 input.parse().map(ForeignItem::Static)
@@ -1270,7 +1324,21 @@ pub mod parsing {
                 input.parse().map(ForeignItem::Macro)
             } else {
                 Err(lookahead.error())
+            }?;
+
+            {
+                let item_attrs = match item {
+                    ForeignItem::Fn(ref mut item) => &mut item.attrs,
+                    ForeignItem::Static(ref mut item) => &mut item.attrs,
+                    ForeignItem::Type(ref mut item) => &mut item.attrs,
+                    ForeignItem::Macro(ref mut item) => &mut item.attrs,
+                    ForeignItem::Verbatim(_) => unreachable!(),
+                };
+                attrs.extend(item_attrs.drain(..));
+                *item_attrs = attrs;
             }
+
+            Ok(item)
         }
     }
 
@@ -1639,11 +1707,11 @@ pub mod parsing {
 
     impl Parse for TraitItem {
         fn parse(input: ParseStream) -> Result<Self> {
+            let mut attrs = input.call(Attribute::parse_outer)?;
             let ahead = input.fork();
-            ahead.call(Attribute::parse_outer)?;
 
             let lookahead = ahead.lookahead1();
-            if lookahead.peek(Token![const]) {
+            let mut item = if lookahead.peek(Token![const]) {
                 ahead.parse::<Token![const]>()?;
                 let lookahead = ahead.lookahead1();
                 if lookahead.peek(Ident) {
@@ -1673,7 +1741,21 @@ pub mod parsing {
                 input.parse().map(TraitItem::Macro)
             } else {
                 Err(lookahead.error())
+            }?;
+
+            {
+                let item_attrs = match item {
+                    TraitItem::Const(ref mut item) => &mut item.attrs,
+                    TraitItem::Method(ref mut item) => &mut item.attrs,
+                    TraitItem::Type(ref mut item) => &mut item.attrs,
+                    TraitItem::Macro(ref mut item) => &mut item.attrs,
+                    TraitItem::Verbatim(_) => unreachable!(),
+                };
+                attrs.extend(item_attrs.drain(..));
+                *item_attrs = attrs;
             }
+
+            Ok(item)
         }
     }
 
@@ -1734,8 +1816,8 @@ pub mod parsing {
                 attrs: private::attrs(outer_attrs, inner_attrs),
                 sig: MethodSig {
                     constness: constness,
-                    unsafety: unsafety,
                     asyncness: None,
+                    unsafety: unsafety,
                     abi: abi,
                     ident: ident,
                     decl: FnDecl {
@@ -1823,7 +1905,7 @@ pub mod parsing {
             let outer_attrs = input.call(Attribute::parse_outer)?;
             let defaultness: Option<Token![default]> = input.parse()?;
             let unsafety: Option<Token![unsafe]> = input.parse()?;
-            let impl_token: Token![impl ] = input.parse()?;
+            let impl_token: Token![impl] = input.parse()?;
 
             let has_generics = input.peek(Token![<])
                 && (input.peek2(Token![>])
@@ -1839,6 +1921,7 @@ pub mod parsing {
             };
 
             let trait_ = {
+                // TODO: optimize using advance_to
                 let ahead = input.fork();
                 if ahead.parse::<Option<Token![!]>>().is_ok()
                     && ahead.parse::<Path>().is_ok()
@@ -1883,8 +1966,8 @@ pub mod parsing {
 
     impl Parse for ImplItem {
         fn parse(input: ParseStream) -> Result<Self> {
+            let mut attrs = input.call(Attribute::parse_outer)?;
             let ahead = input.fork();
-            ahead.call(Attribute::parse_outer)?;
             let vis: Visibility = ahead.parse()?;
 
             let mut lookahead = ahead.lookahead1();
@@ -1896,7 +1979,7 @@ pub mod parsing {
                 None
             };
 
-            if lookahead.peek(Token![const]) {
+            let mut item = if lookahead.peek(Token![const]) {
                 ahead.parse::<Token![const]>()?;
                 let lookahead = ahead.lookahead1();
                 if lookahead.peek(Ident) {
@@ -1935,7 +2018,22 @@ pub mod parsing {
                 input.parse().map(ImplItem::Macro)
             } else {
                 Err(lookahead.error())
+            }?;
+
+            {
+                let item_attrs = match item {
+                    ImplItem::Const(ref mut item) => &mut item.attrs,
+                    ImplItem::Method(ref mut item) => &mut item.attrs,
+                    ImplItem::Type(ref mut item) => &mut item.attrs,
+                    ImplItem::Existential(ref mut item) => &mut item.attrs,
+                    ImplItem::Macro(ref mut item) => &mut item.attrs,
+                    ImplItem::Verbatim(_) => unreachable!(),
+                };
+                attrs.extend(item_attrs.drain(..));
+                *item_attrs = attrs;
             }
+
+            Ok(item)
         }
     }
 
@@ -1962,8 +2060,8 @@ pub mod parsing {
             let vis: Visibility = input.parse()?;
             let defaultness: Option<Token![default]> = input.parse()?;
             let constness: Option<Token![const]> = input.parse()?;
-            let unsafety: Option<Token![unsafe]> = input.parse()?;
             let asyncness: Option<Token![async]> = input.parse()?;
+            let unsafety: Option<Token![unsafe]> = input.parse()?;
             let abi: Option<Abi> = input.parse()?;
             let fn_token: Token![fn] = input.parse()?;
             let ident: Ident = input.parse()?;
@@ -1987,8 +2085,8 @@ pub mod parsing {
                 defaultness: defaultness,
                 sig: MethodSig {
                     constness: constness,
-                    unsafety: unsafety,
                     asyncness: asyncness,
+                    unsafety: unsafety,
                     abi: abi,
                     ident: ident,
                     decl: FnDecl {
@@ -2153,8 +2251,8 @@ mod printing {
             tokens.append_all(self.attrs.outer());
             self.vis.to_tokens(tokens);
             self.constness.to_tokens(tokens);
-            self.unsafety.to_tokens(tokens);
             self.asyncness.to_tokens(tokens);
+            self.unsafety.to_tokens(tokens);
             self.abi.to_tokens(tokens);
             NamedDecl(&self.decl, &self.ident).to_tokens(tokens);
             self.block.brace_token.surround(tokens, |tokens| {
@@ -2600,8 +2698,8 @@ mod printing {
     impl ToTokens for MethodSig {
         fn to_tokens(&self, tokens: &mut TokenStream) {
             self.constness.to_tokens(tokens);
-            self.unsafety.to_tokens(tokens);
             self.asyncness.to_tokens(tokens);
+            self.unsafety.to_tokens(tokens);
             self.abi.to_tokens(tokens);
             NamedDecl(&self.decl, &self.ident).to_tokens(tokens);
         }
