@@ -178,6 +178,19 @@
 //!    |                ^^^
 //! ```
 //!
+//! ## Testing
+//!
+//! When testing macros, we often care not just that the macro can be used
+//! successfully but also that when the macro is provided with invalid input it
+//! produces maximally helpful error messages. Consider using the [`trybuild`]
+//! crate to write tests for errors that are emitted by your macro or errors
+//! detected by the Rust compiler in the expanded code following misuse of the
+//! macro. Such tests help avoid regressions from later refactors that
+//! mistakenly make an error no longer trigger or be less helpful than it used
+//! to be.
+//!
+//! [`trybuild`]: https://github.com/dtolnay/trybuild
+//!
 //! ## Debugging
 //!
 //! When developing a procedural macro it can be helpful to look at what the
@@ -222,7 +235,8 @@
 //!   dynamic library libproc_macro from rustc toolchain.
 
 // Syn types in rustdoc of other crates get linked to here.
-#![doc(html_root_url = "https://docs.rs/syn/0.15.26")]
+#![doc(html_root_url = "https://docs.rs/syn/0.15.39")]
+#![allow(unknown_lints, bare_trait_objects, ellipsis_inclusive_range_patterns)]
 #![cfg_attr(feature = "cargo-clippy", allow(renamed_and_removed_lints))]
 #![cfg_attr(feature = "cargo-clippy", deny(clippy, clippy_pedantic))]
 // Ignored clippy lints.
@@ -230,8 +244,7 @@
     feature = "cargo-clippy",
     allow(
         block_in_if_condition_stmt,
-        const_static_lifetime,
-        cyclomatic_complexity,
+        cognitive_complexity,
         deprecated_cfg_attr,
         doc_markdown,
         eval_order_dependence,
@@ -239,6 +252,7 @@
         needless_pass_by_value,
         never_loop,
         redundant_field_names,
+        redundant_static_lifetimes,
         too_many_arguments,
     )
 )]
@@ -538,7 +552,9 @@ pub use gen::*;
 #[doc(hidden)]
 pub mod export;
 
-mod keyword;
+mod custom_keyword;
+mod custom_punctuation;
+mod sealed;
 
 #[cfg(feature = "parsing")]
 mod lookahead;
@@ -672,7 +688,8 @@ pub fn parse_str<T: parse::Parse>(s: &str) -> Result<T> {
 ///
 /// If present, either of these would be an error using `from_str`.
 ///
-/// *This function is available if Syn is built with the `"parsing"` and `"full"` features.*
+/// *This function is available if Syn is built with the `"parsing"` and
+/// `"full"` features.*
 ///
 /// # Examples
 ///
